/*
 * NEOpixel.c
 *
 *  Created on: 8. 5. 2021
 *      Author: honzi
 */

#include "NEOpixel.h"
volatile uint32_t pwmVal = 0;   // 32b protoze i CCRx je 32b
volatile uint32_t pwmBits[NUM_BITS];
volatile uint32_t curBit = 0;
volatile uint32_t numPixels = 0;


void TIM3_IRQHandler(void)
{
  TIM3->SR &= ~TIM_SR_UIF;  //
//  TIM3->SR &= ~TIM_SR_TIF;  //  kterej vlastne prisel ?

  GPIOToggle(GPIOA, 5);

  if (curBit >= (24 * numPixels))
  {
    TIM3->CCMR1 &= ~TIM_CCMR1_OC2M_1;
    TIM3->CR1 &= ~TIM_CR1_CEN;
  }
  else
  {
    if (curBit == 0)
      TIM3->CCMR1 |= TIM_CCMR1_OC2M_1;

    TIM3->CCR2 = pwmBits[curBit];
    curBit++;
  }
}

bool NEOpixelInit(uint32_t pocetRGB)
{
  int apb = GetTimerClock(3);

  if (!(RCC->APB1ENR & RCC_APB1ENR_TIM3EN))
  {
    RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;
    RCC->APB1RSTR |= RCC_APB1RSTR_TIM3RST;
    RCC->APB1RSTR &= ~RCC_APB1RSTR_TIM3RST;
  }

  TIM3->CR1 = 0;   // ne TIM_CR1_URS;, pujde startnout pomoci EGR_UG
  TIM3->CR2 = 0;
  TIM3->PSC = apb / 1E7 - 1;  // 0.1us = 10MHz
  TIM3->ARR = PWM_BIT_LEN - 1;         // reload pro 1.2 us

  TIM3->DIER |= TIM_DIER_UIE;
  NVIC_EnableIRQ(TIM3_IRQn);  // hledat podle typu ve volani teto funkce

  TIM3->CCMR1 &= ~TIM_CCMR1_OC2M;
  TIM3->CCMR1 |= TIM_CCMR1_OC2M_2;  // 100 = Force inactive leve

  TIM3->CCER |= TIM_CCER_CC2E;
  TIM3->CCR2 = 0;  // TIM3->ARR / 2;

  Nucleo_SetPinGPIO(GPIOB, 5, ioPortAlternatePP);  // Arduino D4
  Nucleo_SetAFGPIO(GPIOB, 5, 2);    // AF02 = TIM3CH2

  if (pocetRGB > NUM_PIXELS_MAX)
    return false;

  numPixels = pocetRGB;

  return true;
}

bool SetPixel(uint pix, uint8_t r, uint8_t g, uint8_t b)
{
  int i;
  uint32_t volatile *ptr;

  if (pix >= NUM_PIXELS_MAX)        // array limit
    return false;

  if (pix >= numPixels)
    return false;               // logic limit

  ptr = pwmBits + 24 * pix;
  for (i = 0; i < 8; i++)
  {
    *ptr = (g & 0x80) ? PWM_BIT_1 : PWM_BIT_0;

    ptr++;
    g <<= 1;
  }

  for (i = 0; i < 8; i++)
  {
    *ptr = (r & 0x80) ? PWM_BIT_1 : PWM_BIT_0;

    ptr++;
    r <<= 1;
  }

  for (i = 0; i < 8; i++)
  {
    *ptr = (b & 0x80) ? PWM_BIT_1 : PWM_BIT_0;

    ptr++;
    b <<= 1;
  }
  return true;
}

void ShiftPixels(int pole[],bool jasDOWN,int i, bool HSLon, bool Ron, bool Gon, bool Bon)
{
  rgb poleBarevRGB;
  int r = 0, g = 0, b = 0;

  for(int x = 0; x<numPixels; x++)
  {
    SetPixel(i,0,0,0);
    if (HSLon)
    {
      poleBarevRGB = hsl2rgb(pole[x]);

      if(jasDOWN)
        SetPixel(x+i,poleBarevRGB.r/10,poleBarevRGB.g/10,poleBarevRGB.b/10);    //Ochrana o...
      else
        SetPixel(x+i,poleBarevRGB.r,poleBarevRGB.g,poleBarevRGB.b);
    }
    else
    {
      if (Ron)
        r = pole[x];
      if (Gon)
        g = pole[x];
      if (Bon)
        b = pole[x];

      if(jasDOWN)
        SetPixel(x+i,r/10,g/10,b/10);    //Ochrana o...
      else
        SetPixel(x+i,r,g,b);
    }

    curBit = 0;
    TIM3->CNT = TIM3->ARR;        // ihned to pretece (doufam)
    TIM3->CR1 |= TIM_CR1_CEN;
  }
}



bool Zobraz(int pole[], bool jasDOWN, bool HSLon, bool Ron, bool Gon, bool Bon)
{
  int r = 0;
  int g = 0;
  int b = 0;

  rgb poleBarevRGB;
  for(int x = 0; x<numPixels ;x++)
  {
    if (HSLon)
    {
      poleBarevRGB = hsl2rgb(pole[x]);

      if(jasDOWN)
        SetPixel(x,poleBarevRGB.r/10,poleBarevRGB.g/10,poleBarevRGB.b/10);    //Ochrana o...
      else
        SetPixel(x,poleBarevRGB.r,poleBarevRGB.g,poleBarevRGB.b);
    }
    else
    {
      if (Ron)
        r = pole[x];
      if (Gon)
        g = pole[x];
      if (Bon)
        b = pole[x];

      if(jasDOWN)
        SetPixel(x,r/10,g/10,b/10);    //Ochrana o...
      else
        SetPixel(x,r,g,b);
    }

    curBit = 0;
    TIM3->CNT = TIM3->ARR;        // ihned to pretece (doufam)
    TIM3->CR1 |= TIM_CR1_CEN;
  }
  return true;
}

