/*
 * RGB_HSL.c
 *
 *  Created on: 7. 5. 2021
 *      Author: honzi
 */

#include "RGB_HSL.h"

hsl rgb2hsl(rgb in)
{
    hsl         out;
    double      min, max, delta;

    min = in.r < in.g ? in.r : in.g;
    min = min  < in.b ? min  : in.b;

    max = in.r > in.g ? in.r : in.g;
    max = max  > in.b ? max  : in.b;

    out.l = max;                                // v
    delta = max - min;
    if (delta < 0.00001)
    {
        out.s = 0;
        out.h = 0; // undefined, maybe nan?
        return out;
    }
    if( max > 0.0 ) { // NOTE: if Max is == 0, this divide would cause a crash
        out.s = (delta / max);                  // s
    } else {
        // if max is 0, then r = g = b = 0
        // s = 0, h is undefined
        out.s = 0.0;
        //out.h = 0;                            // its now undefined
        return out;
    }
    if( in.r >= max )                           // > is bogus, just keeps compilor happy
        out.h = ( in.g - in.b ) / delta;        // between yellow & magenta
    else
    if( in.g >= max )
        out.h = 2.0 + ( in.b - in.r ) / delta;  // between cyan & yellow
    else
        out.h = 4.0 + ( in.r - in.g ) / delta;  // between magenta & cyan

    out.h *= 60.0;                              // degrees

    if( out.h < 0.0 )
        out.h += 360.0;

    return out;
}

rgb hsl2rgb(int hodnota)
{
  rgb poleB;
  double v;
  double r, g, b;

  double sl = 1;
  double l = 0.5;
  double h = (double)hodnota/255/1.5;           //h[0,1] hodnota[0,255] -> hodnota/255 1,5 jsem pidal protoe posledn barvy HSL by nesedli do chtnho spektra, 0-255 = rozsah hodnot

  r = g = b = l;
  v = (l <= 0.5) ? (l * (1.0 + sl)) : (l + sl - l * sl);
  if (v > 0)
  {
      double m;
      double sv;
      int sextant;
      double fract, vsf, mid1, mid2;
      m = l + l - v;
      sv = (v - m) / v;
      h *= 6.0;
      sextant = (int)h;
      fract = h - sextant;
      vsf = v * sv * fract;
      mid1 = m + vsf;
      mid2 = v - vsf;
      switch (sextant)
      {
          case 0:
              r = v;
              g = mid1;
              b = m;
              break;

          case 1:
              r = mid2;
              g = v;
              b = m;
              break;

          case 2:
              r = m;
              g = v;
              b = mid1;
              break;
          case 3:
              r = m;
              g = mid2;
              b = v;
              break;
          case 4:
              r = mid1;
              g = m;
              b = v;
              break;
          case 5:
              r = v;
              g = m;
              b = mid2;
              break;
      }
  }
  poleB.r = r * 255;
  poleB.g = g * 255;
  poleB.b = b * 255;
  return poleB;

}
